package de.hamster.debugger.model;

/*lego*/import de.hamster.lego.model.LHamster;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintStream;
import de.hamster.workbench.Utils;

import de.hamster.model.HamsterProgram;
import de.hamster.model.InstructionProcessor;

/**
 * Die Klasse hat zwei Funktionen. Zum einen wird ihre main-Methode bei der
 * Ausfuehrung eines Hamsterprogramms in der Client-Virtual-Maschine aufgerufen.
 * Zum anderen erzeugt sie den Simulator-seitigen RemoteProcessor und startet
 * und stoppt diesen.
 * 
 * @author Daniel Jasper
 */
public class RemoteRunner {
	/**
	 * Dies ist der Simulator-seitige RemoteProcessor
	 */
	private RemoteProcessor remoteProcessor;

	/**
	 * Mit diesem Konstruktor wird der RemoteRunner auf Server-Seite erzeugt.
	 * Dieser erzeugt direkt den serverseitigen RemoteProcessor.
	 * 
	 * @param processor
	 *            An diesen Processor muessen die vom Client kommenden Befehle
	 *            weitergeleitet werden
	 * @param p
	 *            Dies ist der Prozess, in dem die Client-VM lauuft. Ueber
	 *            dieses Objekt koennen die Ein- und Ausgabestroeme verbunden
	 *            werden.
	 */
	public RemoteRunner(InstructionProcessor processor, Process p) {
		remoteProcessor = new RemoteProcessor(p.getInputStream(), p
				.getOutputStream(), processor);
	}

	/**
	 * Diese Methode stoppt den RemoteProcessor
	 */
	public void stop() {
		remoteProcessor.terminate();
	}

	/**
	 * Diese Methode startet den Simulator-seitigen RemoteProcessor, wodurch der
	 * Simulator dann die Befehle des Clients empfangen kann.
	 */
	public void start() {
		Thread thread = new Thread(remoteProcessor);
		thread.start();
	}

	/**
	 * Leitet RemoteProcessor.setDelay(int delay) weiter.
	 * 
	 * @param delay
	 *            Die Verzoegerung nach der Ausfuehrung jedes Befehls.
	 */
	public void setDelay(int delay) {
		remoteProcessor.setDelay(delay);
	}

	/**
	 * Dies ist die main-Methode, die aufgerufen wird, wenn ein Hamsterprogramm
	 * in einer Client-VM ausgefuehrt wird.
	 * 
	 * @param args
	 *            Es wird ein Parameter uebergeben, der Name des auszufuehrenden
	 *            Hamsterprogramms
	 */
	public static void main(String[] args) {
		try {
			RemoteProcessor rp = new RemoteProcessor(System.in, System.out);
			System.setOut(new PrintStream(new FileOutputStream("sysout.txt")));
			System.setErr(new PrintStream(new FileOutputStream("syserr.txt")));
			/* lego */LHamster.processor = rp;
			IHamster.processor = rp;
			Territorium.processor = rp;
			String className = args[0];
			Object hamsterObject;
			Utils.loadProperties(); // dibo
			try {
				Class hamsterClass = Class.forName(className);
				hamsterObject = hamsterClass.newInstance();
				HamsterProgram hamster = (HamsterProgram) hamsterObject;
				Hamster.getStandardHamster();
				if (Utils.security) { // dibo
					System.setSecurityManager(new SecurityManager());
				}
				hamster.main();
			} catch (Exception e) {
				rp.processException(e);
				e.printStackTrace();
			} catch (Error e) {
				rp.processException(new RuntimeException(e));
				e.printStackTrace();
			}
		} catch (IOException e1) {
			e1.printStackTrace();
		}
	}
}
